/*

This function checks if the BadBlock Table is existing in the Flash
by reading first two bytes. These two bytes should contain either 
0xA1 (indicating valid block) or 0xBD (indicating Bad Block)

If table exists in the flash, it is copied to BadBlockTable array.
Else, the function detects the blocks which are Bad and keeps 
that information in the flash. 

Parameter to be passed: BlockNo
Returns: None

*/


#include "Extern_Declaration.h"

/**********************************************************************/

void CheckTableExist(unsigned int Cblock)
{

	// Wait till write buffer is empty:
	
	while(!(*pNFC_STAT & WB_EMPTY));
	
	// Page Read Command:
	
	*pNFC_CMD =  PageReadCmd[0];
	 
	/*  
	
	Set up the address cycles depending on Block number passed:
	
	Column: A0-A11
	Page: 	A12-A17
	Block: 	A18-A28
	
	
	1st Address Cycle: A0-A7
	2nd Address Cycle: A8-A11
	3rd Address Cycle: A12-A19
	4th Address Cycle: A20-A27
	5th Address Cycle: A28
	
	*/
	
	short temp_Block = Cblock;
	short temp_Page = 0;

	//Column
	*pNFC_ADDR = 0;
	*pNFC_ADDR = 0;
	
	//Page
	
	
	temp_Page = Cblock & 3;		// Get bits A18 and A19 from Block address
	temp_Page = temp_Page << 6;	// Shift to append with Page Address which is 0
	
	// First 2 bits of Block and 6 bits of Page are sent which is 0
	*pNFC_ADDR = temp_Page;	
	
	// Wait till write buffer is empty:
	
	while(!(*pNFC_STAT & WB_EMPTY));
	
	// Next 8 bits of Block address. i.e till 1023. 
			
	Cblock = temp_Block;
	Cblock = Cblock & 0x3FC;	// Mask to get A20-A27	
	Cblock = Cblock >> 2;		// Because A18 and A19 are already sent out
	
	*pNFC_ADDR = Cblock;
	
		
	// Block address greater than 1023
	
	Cblock = temp_Block;
	Cblock = Cblock & 0xFC00;	// Mask to get A28-A30
	Cblock = Cblock >> 10;		// Because A18-A27 are already sent out
	
	*pNFC_ADDR = Cblock;
	

	// Next command
	
	*pNFC_CMD =  PageReadCmd[1];
	ssync();
	
	//Wait till NBUSY_Flag sets here.
	
	while(!NBUSY_Flag); 
	NBUSY_Flag = 0;
	
/* 

Here read two bytes of Page0 to determine if the BadBlock Table is 
residing in Flash. If it is already stored, the first two bytes
will either be 0xA1 or 0xBD

*/
 
	short Read_Data1, Read_Data2 ;

	//trigger a read request to nand flash.
		
	*pNFC_DATA_RD = 0;
	while(!(*pNFC_IRQSTAT & RD_RDY));
	Read_Data1 = *pNFC_READ;
	*pNFC_IRQSTAT |= RD_RDY;
	ssync();
	
	//trigger a read request to nand flash.
		
	*pNFC_DATA_RD = 0;
	while(!(*pNFC_IRQSTAT & RD_RDY));
	Read_Data2 = *pNFC_READ;
	*pNFC_IRQSTAT |= RD_RDY;
	ssync();

	if(((Read_Data1 != 0xA1) & (Read_Data1 != 0xBD)) & ((Read_Data2 != 0xA1) & (Read_Data2 != 0xBD)))
	{
	
		for(i = 0; i<Flash_Blocks; i++)
		{
			DetectBadBlock(i);
		}
			
		printf("Bad Block Detection Completed and is stored in Block %d\n", ResBlockNo);
		BlockErase(ResBlockNo);
		PageWrite(0, ResBlockNo, &BadBlockTable[0]);
	}

	else
	
	{
		// Bad Block is stored in flash. Read it into internal memory
		PageRead(0, ResBlockNo, &BadBlockTable[0]);
		printf("Bad Block is stored in Block %d\n", ResBlockNo);
		
	}
		
}

	
/**********************************************************************/



